﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private Steema.TeeChart.Styles.Surface surface;
        private Steema.TeeChart.Tools.ChartImage colorGridImage = new Steema.TeeChart.Tools.ChartImage();
        double Avarage;
        double[,] data = null;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            surface = new Steema.TeeChart.Styles.Surface();
            surface.UseColorRange = false;
            surface.UsePalette = true;
            surface.Pen.Visible = false;
            tChart1.Series.Add(surface);
            tChart1.Tools.Add(colorGridImage);
            surface.Visible = true;
            changeViewMode(true);
            ClearSafe();
            ScaleColorGrid(1000, 50);
            showData();
            timer1.Enabled = true;
        }

        private void showData()
        {          
            if(data ==null)
            {
                //??data = readData(@"D:\Mahesh\testData.csv");
                data = readData(Path.Combine(Environment.CurrentDirectory, @"..\..\testData.csv"));// \testData.csv");
            }
            tChart1.AutoRepaint = false;

            for (int i = 0; i < data.GetUpperBound(0) + 1; i++)
            {
                for (int j = 0; j < data.GetUpperBound(1) + 1; j++)
                {
                    if (double.IsNaN(data[i, j]))
                    {
                        surface.Add(j, Avarage, i, Color.Transparent);
                    }
                    else
                    {
                        surface.Add(j, data[i, j], i);
                    }
                }
            }
            tChart1.AutoRepaint = true;
            tChart1.Refresh();
        }

        private double[,] readData(string path)
        {
            double[,] data=new double[50,1000];

            char[] specialChars = { ',', '\"', '\n', '\r' };
            System.IO.StreamReader reader = new System.IO.StreamReader(path);

            string line = "";
            int i=0;
            int count = 0;
            double sum = 0;
            while (!reader.EndOfStream)
            {
                line = reader.ReadLine();
                string[] columns = line.Split(new char[] { ',' },StringSplitOptions.RemoveEmptyEntries);
                for (int j = 0; j < columns.Length; j++)
                {
                    data[i, j] = Convert.ToDouble(columns[j]);
                    if (!double.IsNaN(data[i, j]))
                    {
                        count++;
                        sum += data[i, j];
                    }
                }
                i++;
            }
            reader.Close();
            reader.Dispose();
            Avarage = sum / count;
            return data;
        }

        private void ClearSafe()
        {
            try
            {
                surface.Clear();
                colorGridImage.Image = null;               
         
                surface.PaletteSteps = 30;
                surface.ClearPalette();
                surface.UseColorRange = false;
                surface.StartColor = Color.Green;
                surface.MidColor = Color.LightGreen;
                surface.EndColor = Color.White;
                surface.UsePalette = true;
                tChart1.Refresh();
                surface.UsePalette = true;
            }
            catch (Exception ex)
            {
               
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ClearSafe();
            surface.Clear();
            ScaleColorGrid(1000,50);
            tChart1.AutoRepaint = false;
            tChart1.Axes.Left.Maximum = Convert.ToDouble( txtMax.Text);
            tChart1.Axes.Left.Minimum = Convert.ToDouble(txtMin.Text);
            
            Steema.TeeChart.Axis scanAxis = tChart1.Axes.Depth;
            scanAxis.Automatic = false;
            scanAxis.Minimum = 1;
            scanAxis.Maximum = 50 + 1;
            scanAxis.Inverted = true;

            tChart1.Axes.Bottom.Automatic = false;
            tChart1.Axes.Bottom.Minimum = 1;
            tChart1.Axes.Bottom.Maximum = 1000 + 1;

            showData();
            tChart1.AutoRepaint = true;
            tChart1.Refresh();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DisposeObject(surface); surface = null;
            DisposeObject(colorGridImage); colorGridImage = null;
        }
        private void DisposeObject(object o)
        {
            try
            {
                if (o != null)
                {
                    if (o is IDisposable)
                        ((IDisposable)o).Dispose();
                }
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.ToString());
            }
        }
        private void ScaleColorGrid(int maxX, int maxY)
        {
            try
            {  
                tChart1.AutoRepaint = false;

                Steema.TeeChart.Axis scanAxis = tChart1.Axes.Left;
                scanAxis = tChart1.Axes.Depth;

                scanAxis.Automatic = false;
                scanAxis.Minimum = 1;
                scanAxis.Maximum = maxY + 1;
                scanAxis.Inverted = true;

                tChart1.Axes.Bottom.Automatic = false;
                tChart1.Axes.Bottom.Minimum = 1;
                tChart1.Axes.Bottom.Maximum = maxX + 1;

                tChart1.AutoRepaint = true;  
            }
            catch (Exception ex)
            {
                
            }
        }
        private void changeViewMode(bool mode3D)        
        {       
            if (tChart1.Zoom.Zoomed)
                tChart1.Zoom.Undo();
            tChart1.Panning.Allow = Steema.TeeChart.ScrollModes.None;
            tChart1.Aspect.View3D = mode3D;

            tChart1.Axes.Bottom.Title.Text = "Bin";
            tChart1.Axes.Bottom.Title.Visible = mode3D;
            tChart1.Axes.Bottom.Labels.Visible = mode3D;

            tChart1.Axes.Depth.Automatic = false;
            tChart1.Axes.Depth.Maximum = 50;
            tChart1.Axes.Depth.Visible = mode3D;
            tChart1.Axes.Depth.Title.Text = "Scan";
            tChart1.Axes.Depth.Title.Visible = mode3D;
            tChart1.Axes.Depth.Labels.Visible = mode3D;
            tChart1.Axes.Depth.Inverted = mode3D;
       
            tChart1.Axes.Left.Automatic = !mode3D;
            tChart1.Axes.Left.Visible = mode3D;
            tChart1.Axes.Left.Labels.Visible = mode3D;

            tChart1.Axes.Bottom.Labels.Font.Size += 10;
            tChart1.Axes.Bottom.Title.Font.Size += 10;
            tChart1.Axes.Depth.Labels.Font.Size += 10;
            tChart1.Axes.Depth.Title.Font.Size += 10;
            tChart1.Axes.Left.Labels.Font.Size += 10;
            tChart1.Axes.Left.Title.Font.Size += 10; 
         
            tChart1.Panning.Allow = Steema.TeeChart.ScrollModes.None;


            if (mode3D)
            {
                tChart1.Aspect.Chart3DPercent = 90;
                tChart1.Aspect.Elevation = 348;
                tChart1.Aspect.Orthogonal = false;
                tChart1.Aspect.Perspective = 62;
                tChart1.Aspect.Rotation = 329;
                tChart1.Aspect.VertOffset = -20;
                tChart1.Aspect.Zoom = 60;                                 
            }
            else
            {
                tChart1.Aspect.Zoom = 100;              
            }

            tChart1.Refresh();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            showData();
            this.Refresh();
        }
    }
}
